"""
    Pre-Schema
"""

import strawberry
from graphql.validation import NoSchemaIntrospectionCustomRule
from strawberry.extensions import AddValidationRules, QueryDepthLimiter


def Schema(
    query: object = None,
    mutation: object = None,
    extensions: list = None,
    max_depth: int = 4,
    introspection: bool = True,
    **kwargs
) -> strawberry.Schema | None:
    """Strawberry Schema Wrapper"""

    query = query or []
    mutation = mutation or []
    extensions = extensions or []

    Extensions = [
        QueryDepthLimiter(max_depth=max_depth),
    ]
    Extensions.extend(extensions)

    # Apps Extensions
    """
    settings = Settings()
    if len(settings.extensions) > 0:
        Extensions.extend(settings.extensions)
    """

    # Introspection
    if not introspection:
        Extensions.append(AddValidationRules([NoSchemaIntrospectionCustomRule]))

    # Query & Mutation
    items = {}
    if query:
        items["query"] = query
    if mutation:
        items["mutation"] = mutation

    # Return Value
    if query:
        return strawberry.Schema(**items, extensions=Extensions, **kwargs)
    return None



import click

import spoc

from .shell import shell_command, shell_print


@click.command()
@click.option("-p", "--port", default=8000, help="Port Number.")
@click.option("-w", "--workers", default=1, help="Number of Workers.")
def run(port, workers):
    """Start { FastBerry } Server

    \b
    # Examples
        - development          : `./manage.py run --port 8000`
        - production & staging : `./manage.py run --port 8000 --workers 4`
    """
    mode = spoc.mode
    # FastAPI-Server
    match mode:
        case "development":
            run_server = f"python -m uvicorn main:app --reload --port { port }"
        case "staging":
            run_server = f"python -m gunicorn main:app --workers { workers } --worker-class uvicorn.workers.UvicornWorker --bind 0.0.0.0:{ port }"
        case "production":
            run_server = f"python -m gunicorn main:app --workers { workers } --worker-class uvicorn.workers.UvicornWorker --bind 0.0.0.0:{ port }"

    # Message
    shell_print(f"* Starting FastApi Server... (Mode: { mode.title() })\n")

    # RETURN-VALUE
    shell_command(run_server)



import typing

from ..core import types


def api_info(
    base_dir: typing.Any = None, pyproject: dict = None, toml: typing.Any = None
):
    """API's Info"""
    current_path = base_dir
    current_info = pyproject.get("project", {})
    # Get { Docs }
    app_description = None
    docs_file = toml.get("spoc", {}).get("docs")
    if docs_file:
        for path in docs_file.split("/"):
            current_path /= path
        if current_path.exists():
            with open(current_path, "r", encoding="utf-8") as file:
                app_description = file.read()
    # FastAPI { INFO }
    return types.FastAPI(
        title=current_info.get("name", "fastberry"),
        version=current_info.get("version", "0.1.0"),
        description=app_description,
    )
